# draw_fig3.R
#
# Jari Vauhkonen
# 4 July 2018
#
# This R-script can be used to visualize results as in Figure 3 of the comment paper;
#
# Note: for ease, I assume that previous analysis scripts were run earlier and the resulting R-objects were stored to files which are loaded below.
# The code below can be executed this way, or running the earlier scripts subsequently and commenting off the statements cleaning the workspace and loading earlier objects
#

rm(list=ls())

# Load values for the other template database...
load("nini_1000.RData")

# Store values locally
inds.sp.t1.1000 <- inds.sp.t1
si.sps.1000 <- si.sps
nini.sps.1000 <- nini.sps
sp.h.curves.1000 <- sp.h.curves
sp.g.curves.1000 <- sp.g.curves
sp.v.curves.1000 <- sp.v.curves

# ... load the value for the another DB
load("nini_100.RData")

# Store values locally
inds.sp.t1.100 <- inds.sp.t1
si.sps.100 <- si.sps
nini.sps.100 <- nini.sps
sp.h.curves.100 <- sp.h.curves
sp.g.curves.100 <- sp.g.curves
sp.v.curves.100 <- sp.v.curves

tiff("Fig3.tif", width=190, height=90, units="mm", res=300, pointsize=10)
par(mfrow=c(1,3), mar=c(5,4,1,1))

# Species index pointing to the following vector of dominant species : "aw" "pl" "sb" "sw"
# For visualizing, I use of constant value of 1 producing the results for AW
sp <- 1

# Loop over the three column indices used to store the attributes height, basal area, and volume (first index is the dominant species)
for (nro in 2:4) {
  # Plotting conventions
  maxs <- c(0,34,35,380)
  mins <- c(0,18,26,225)
  plot(range(stand.age), c(mins[nro],maxs[nro]), type="n", xlab="Age, years", ylab="", cex.axis=1.5, cex.lab=1.5)
  if (nro==2) {
    text(5, 34, "(a)", cex=1.5)
    mtext("Height, m", 2, 2.5, cex=1)
  } else if (nro==3) {
    text(5, 35, "(b)", cex=1.5)
    mtext(expression(paste("Basal area,  ", m^2/ha)), 2, 2.5, cex=1)
  } else {
    text(5, 380, "(c)", cex=1.5)
    mtext(expression(paste("Volume,  ", m^3/ha)), 2,2.5, cex=1 )
  } 

  # Clumsy way to loop over the two databases - use img index to point to one of them at the time
  for (img in 1:2) {
    if (img==1) {
      inds.sp.t1 <- inds.sp.t1.100
      si.sps <- si.sps.100
      nini.sps <- nini.sps.100
      sp.h.curves <- sp.h.curves.100
      sp.g.curves <- sp.g.curves.100
      sp.v.curves <- sp.v.curves.100
    } else {
      inds.sp.t1 <- inds.sp.t1.1000
      si.sps <- si.sps.1000
      nini.sps <- nini.sps.1000
      sp.h.curves <- sp.h.curves.1000
      sp.g.curves <- sp.g.curves.1000
      sp.v.curves <- sp.v.curves.1000
    }
  
    # Extract the curves from the data structures; cf. earlier scripts
    closest.curve.index <- as.numeric(inds.sp.t1[sp,nro])
    if (nro < 3) {
      avg.curve.index <- si.sps[sp]
    } else {
      avg.curve.index <- nini.sps[sp]
    }
    candidate.curves <- list( h=sp.h.curves[sp], g=sp.g.curves[sp], v=sp.v.curves[sp] )
    current.means.t1 <- list( a=a[sp], h=h.t1[sp], g=g.t1[sp], v=v.t1[sp] )
    curves <- candidate.curves[nro-1][[1]][1][[1]]

    # Draw the observation as a point and draw a vertical line from it to the closest template, illustrating the discrepancy from the observed current state to the point from which the growht projection starts
    points(current.means.t1$a, current.means.t1[nro], pch=16)
    lines( rep(current.means.t1$a,2), c(current.means.t1[nro], curves[avg.curve.index,current.means.t1$a]), lwd=1, lty=2, col=2)

    # Prepare the closest template such that it will be visible only from the current time point onwards: mark the values of earlier years as NAs
    range.cur <- 1:(current.means.t1$a-1)
    c <- curves[closest.curve.index,]
    c[range.cur] <- NA
    # Draw the line. Then do the same for the averaged curve
    lines(c, lwd=0.5, lty=2)
    c <- curves[avg.curve.index,]
    c[range.cur] <- NA
    lines(c,lwd=1, col=2)
  }


  # Extend Figure 3 by applying the GYPSY models directly with the input data of the template matching

  # Top height: pick up a curve from the underlying database corresponding to the SI value of the template matching
  hcurves <- candidate.curves[1][[1]][1][[1]]
  hcurve <- hcurves[as.numeric(inds.sp.t1[sp,2]),]
  range.cur <- 1:(current.means.t1$a-1)
  hcurve[range.cur] <- NA

  # Basal area: 
  # generate two sequences, applying the same SI as above, current basal area and Nini as 700 (ba2l) and 6700 (ba2u) stems/ha
  si.cur <- site.index[as.numeric(inds.sp.t1[sp,2])]
  ba2l <- ba2u <- current.means.t1$g
  for (age in current.means.t1$a:max(stand.age)) {
    ba.cur <- ba2l[length(ba2l)]
    ba2l <- c(ba2l, ba.cur + BAINC(dominant.species[sp],si.cur,age,700,ba.cur))
    ba.cur <- ba2u[length(ba2u)]
    ba2u <- c(ba2u, ba.cur + BAINC(dominant.species[sp],si.cur,age,6700,ba.cur))
  }
  # Adjust the values for plotting: remove the first (current) year of the sequence & mark all preceding years as NAs;
  # note: values for different Nini are renamed as 700 (glow) and 6700 (gup) stems/ha
  cu <- cl <- c
  cu[!is.na(c)] <- ba2u[-1]
  cl[!is.na(c)] <- ba2l[-1]
  gup <- cu
  glow <- cl
  gup[range.cur] <- NA
  glow[range.cur] <- NA

  # Volume: Use values of the top height and basal area predicted for each year in the total volume model;
  # Two sequences for the different Nini, 700 (vlow) and 6700 (vup) stems/ha
  vup <- vlow <- c()
  for (i in 1:length(hcurve)) {
    vup <- c(vup, Tvol(species=dominant.species[sp], BA=gup[i], H=hcurve[i]))
    vlow <- c(vlow, Tvol(species=dominant.species[sp], BA=glow[i], H=hcurve[i])) 
  }

  # Draw the lines
  if (nro==2) {
    lines(hcurve, col=4)
  } else if (nro==3) {
    lines(gup, col=4)
    lines(glow, col=4)
  } else {
    lines(vup,col=4)
    lines(vlow,col=4)
  }

  # Add the plot with the total volume value obtained from the GYPSY model:
  # compute the values for the top height at current age and take the basal area from the observations.
  # Use the GYPSY volume model to estimate the vol as a function of Htop & BA and draw this value
  current.h <- as.numeric(hcurves[as.numeric(inds.sp.t1[sp,2]),current.means.t1$a])
  current.g <- current.means.t1$g
  current.v <- Tvol(species=dominant.species[sp], BA=current.g, H=current.h)
  points(current.means.t1$a, current.v, pch=1, col=1)

}

dev.off()

