# generate_templates.R
#
# Jari Vauhkonen
# 4 July 2018
#
# This script is supposed to be run after loading the models in gypsy_models.R.
# The script generates the template curve database by running the models with the inputs specified below.
#

# Create vectors of possible species and sequences of stand ages, site indexes and initial stand densities (latter for the BAINC model)
dominant.species <- c('aw','pl','sb','sw')
stand.age <- seq(1,200,by=1)
site.index <- seq(5,30,by=1)
seq.for.nini <- seq(700,2700,by=100)

# Prepare a list structure to store the species-specific height (h), basal area (g) and volume (v) curve templates
sp.h.curves <- sp.g.curves <- sp.v.curves <- list(aw=c(),pl=c(),sb=c(),sw=c())

# Loop over the sequence of dominant species
for (sp in dominant.species) {
  # Prepare data structures to store results and information on site index & initial density values used
  h.curves <- g.curves <- v.curves <- NULL
  si.values <- nini.values <- c()

  # Loop over the sequence of site indexes
  for (si in site.index) {
    # Heights for the current species (sp) & site index (si) and for the sequence of stand ages
    h.cur <- Htop(sp, si, stand.age)
    ba.curves <- vol.curves <- NULL

    # Loop over the sequence of initial stand densities
    for (Nini in seq.for.nini){
      # Collect the values used to a vector
      nini.values <- c(nini.values, Nini)
      si.values <- c(si.values, si)

      # A block of code to compute total basal area for the sequence of stand ages as a sum of accumulated basal area increment given by the model:
      # Loop over the sequence of ages and compute basal area increment with current species, site index, age, initial density and current basal area (ba.cur);
      # Add current basal area by the increment to get a sequence of basal areas over time
      ba <- c()
      for (age in stand.age) {
        if (length(ba)==0) {
          ba.cur <- 0
        } else {
          ba.cur <- ba[age-1]
        }
        ba <- c(ba, ba.cur + BAINC(sp,si,age,Nini,ba.cur))
      }

      # Store the results;
      # Each row has one basal area sequence over the specified stand ages;
      # The rows result from the different initial density (Nini) values for _one_ site index.
      if (is.null(ba.curves)) {
        ba.curves <- ba
      } else {
        ba.curves <- rbind(ba.curves,ba)
      }
    } # end loop Nini

    # Obtain volume curves using the top height and basal area computed above as arguments of the volume function:
    # Loop over the different sequences of basal areas obtained above
    for (j in 1:dim(ba.curves)[1]) {
      # Current basal area is the j:th sequence
      g.cur <- ba.curves[j,]
      # Obtain volumes for the j:th sequence of basal areas and the one possible sequence of top heights for this site index
      tvols <- c()
      for (k in 1:length(h.cur)) {
        tvols <- c(tvols, Tvol(sp, g.cur[k], h.cur[k]))
      }

      # Store the results similarly as above;
      # Each row is now one _volume_ sequence over the specified stand ages;
      # The rows result from the different initial density (Nini) values to compute basal area for this site index.
      if (is.null(vol.curves)) {
        vol.curves <- tvols
      } else {
        vol.curves <- rbind(vol.curves, tvols)
      }
    } # end loop basal area

    # Store the results for this _site index_:
    # - One sequence of heights
    # - Nini sequences of basal area
    # - Nini sequences of volume (computed using each basal area and height sequence above).
    # Note:
    # Vectors nini.values and si.values are continuously added with the values of Nini and SI to finally trace which combination was used to obtain the results on a given row of this data frame.
    if (is.null(v.curves)) {
      h.curves <- h.cur
      g.curves <- ba.curves
      v.curves <- vol.curves
    } else {
      h.curves <- rbind(h.curves, h.cur)
      g.curves <- rbind(g.curves, ba.curves)
      v.curves <- rbind(v.curves, vol.curves)
    }
  } # end loop site index

  # Store the results to the species-specific list structure - sorry, a clumsy solution & duplicate work compared to above...
  if(sp=='aw') {
    sp.h.curves$aw <- h.curves
    sp.g.curves$aw <- g.curves
    sp.v.curves$aw <- v.curves
  } else if(sp=='pl') {
    sp.h.curves$pl <- h.curves
    sp.g.curves$pl <- g.curves
    sp.v.curves$pl <- v.curves
  } else if(sp=='sb') {
    sp.h.curves$sb <- h.curves
    sp.g.curves$sb <- g.curves
    sp.v.curves$sb <- v.curves
  } else {
    sp.h.curves$sw <- h.curves
    sp.g.curves$sw <- g.curves
    sp.v.curves$sw <- v.curves
  }

} # end loop dominant species


# a test structure for plotting the curves; commented off with purpose

#curves <- sp.v.curves$sw

#plot(range(stand.age), c(0,max(curves,na.rm=T)), type="n")
#for (i in 1:dim(curves)[1]) {
#  lines(curves[i,])
#}
